resource "vsphere_virtual_machine" "k8s_instance_workers" {
  count                  = length(var.k8s_instance_workers)
  name                   = "${element(var.k8s_instance_workers[count.index], 0)}.${element(var.k8s_instance_workers[count.index], 1)}.rl9"
  resource_pool_id       = data.vsphere_resource_pool.pool.id
  datastore_id           = element(var.k8s_instance_workers[count.index], 9)
  folder                 = data.vsphere_folder.k8s.path
  num_cpus               = 3
  num_cores_per_socket   = 1
  memory                 = 8192
  guest_id               = "ubuntu64Guest"
  firmware               = "bios"
  sync_time_with_host    = false
  scsi_type              = "pvscsi"
  enable_logging         = false
  cpu_hot_add_enabled    = true
  cpu_hot_remove_enabled = true
  memory_hot_add_enabled = true

  cdrom {
    client_device = true
  }

  disk {
    label            = "disk0"
    size             = 32
    unit_number      = 0
    eagerly_scrub    = false
    thin_provisioned = true
    keep_on_remove   = false
  }

  clone {
    template_uuid = data.vsphere_virtual_machine.k8s.id
    customize {
      linux_options {
        host_name    = element(var.k8s_instance_workers[count.index], 0)
        domain       = element(var.k8s_instance_workers[count.index], 1)
        hw_clock_utc = true
        time_zone    = "America/Chicago"
      }
      # primary
      network_interface {
        ipv4_address = element(var.k8s_instance_workers[count.index], 2)
        ipv4_netmask = element(var.k8s_instance_workers[count.index], 3)
        ipv6_address = element(var.k8s_instance_workers[count.index], 5)
        ipv6_netmask = element(var.k8s_instance_workers[count.index], 6)
      }
      ipv4_gateway = element(var.k8s_instance_workers[count.index], 4)
      ipv6_gateway = element(var.k8s_instance_workers[count.index], 7)
      dns_server_list = [
        "1.0.0.1",
        "1.1.1.1",
      ]
      dns_suffix_list = [
        element(var.k8s_instance_workers[count.index], 1)
      ]
      # secondary used for NFS
      # network_interface {
      #   ipv4_address = element(var.k8s_instance_workers[count.index], 10)
      #   ipv4_netmask = element(var.k8s_instance_workers[count.index], 11)
      #   ipv6_address = element(var.k8s_instance_workers[count.index], 12)
      #   ipv6_netmask = element(var.k8s_instance_workers[count.index], 13)
      # }
    }
  }

  # primary
  network_interface {
    network_id = data.vsphere_network.exterior.id
  }

  # secondary used for NFS
  # network_interface {
  #   network_id = data.vsphere_network.interior.id
  # }

  connection {
    type        = "ssh"
    agent       = "false"
    host        = element(var.k8s_instance_workers[count.index], 2)
    user        = var.vm_username
    private_key = file("~/.ssh/id_rsa")
  }

  # make script from template
  provisioner "file" {
    destination = "/tmp/provision_items.sh"
    content     = file("${path.module}/provision_items.sh")
  }

  # creates filesystem, installs snmpd, dstat, and more!
  provisioner "remote-exec" {
    inline = [
      "chmod +x /tmp/provision_items.sh",
      "sudo -S /tmp/provision_items.sh >/tmp/provision_items.log 2>&1"
    ]
  }
}
